#include "global.h"
#include "mainmenu.h"
#include "gamedialog.h"
#include "gamewidget.h"

//------------------------------------------------------------------------------------


// Placeholder for dialogs that have not yet been implemented.
class DialogNYI : public GameDialog {
	private:
		GameWidget desktop;
		TextButton bBack;

	public:
		DialogNYI() : GameDialog() {
			desktop.Shape(0, 0, SCREEN_W, SCREEN_H);
			Add(desktop);

			int dy = 12;
			int y = 48-dy;
			int h = 10;
			int x = 10;
			int w = 80;

			bBack.Shape(x, y+=dy, w, h, 1);
			bBack.SetText("Not yet implemented!");
			bBack.MakeExit();
			Add(bBack);
		}

		int Run() {
			Widget *obj = Execute(&bBack);
			return S_MAIN_MENU;
		}
};


//------------------------------------------------------------------------------------

// Now comes the core of our simple game menu system. The system is a finite state
// machine. It has a number of states where each has a dialog attached (except for
// the exit state). At all times the control is at exactly one of the states. For
// every iteration through the FSM the current state is evaluated and control is
// passed to the next state as indicated by the current state's return value.

void GameFramework() {
	// make all our states (dialogs)
	GameDialog *dlg[S_COUNT];
	dlg[S_MAIN_MENU] = new MainMenu;
	dlg[S_OPTIONS] = new DialogNYI;
	dlg[S_HIGHSCORES] = new DialogNYI;
	dlg[S_HELP] = new DialogNYI;
	dlg[S_PLAYING] = new MainGameDialog;
	dlg[S_SECRETGAME] = new DialogNYI;

	// give control to the entry point (the main menu)
	GameDialog *currentDialog = dlg[S_MAIN_MENU];

	// do the state machine in an endless loop
	while (true) {
		// evaluate the current state and see what it wants
		int nextState = currentDialog->Run();

		// if the next state is the exit state, break from the loop
		if (nextState == S_EXIT_GAME) {
			break;
		}
		// otherwise pass the control to the next state
		else {
			currentDialog = dlg[nextState];
		}
	}

	// clean up when we're done
	for (int i=0; i<S_COUNT; i++) {
		delete dlg[i];
	}
}


// the standard MASkinG main function
int main() {
	Error err = InstallMASkinG("allegro.cfg");
	if (err) {
		err.Report();
	}

	set_window_title("Tuna Jump");

	PALETTE pal;
	titlefont = load_font("data/cancunbig.pcx", pal, 0);
	focusfont = load_font("data/cancunfx.pcx", pal, 0);
	normalfont = load_font("data/cancunfx2.pcx", pal, 0);
	set_palette(pal);

	CreateWaves();
	LoadTuna("data/tuna.tga");
	bmpSun = load_bitmap("data/sun.tga", NULL);
	bmpBuoy = load_bitmap("data/buoy.pcx", NULL);
	bmpSign = load_bitmap("data/sign.pcx", NULL);
	bmpRock = load_bitmap("data/rock.pcx", NULL);
	bmpPier = load_bitmap("data/pier.pcx", NULL);
	bmpTroll = load_bitmap("data/troll.pcx", NULL);

	push_config_state();
	set_config_file("allegro.cfg");
	unlockedSecret = get_config_int("TUNA", "secret", 0);
	pop_config_state();

	Cursor cur;
	BITMAP *tmpCur = load_bitmap("data/cur.tga", NULL);
	cur.Create(tmpCur, 4);
	cur.SetFocus(3, 0);
	cur.SetAnimationInterval(100);
	theSkin->SetCursor(cur, Skin::MOUSE_NORMAL);
	destroy_bitmap(tmpCur);

	GameFramework();

	destroy_font(titlefont);
	destroy_font(focusfont);
	destroy_font(normalfont);

	DestroyWaves();
	DestroyTuna();
	destroy_bitmap(bmpSun);
	destroy_bitmap(bmpBuoy);
	destroy_bitmap(bmpSign);
	destroy_bitmap(bmpRock);
	destroy_bitmap(bmpPier);
	destroy_bitmap(bmpTroll);

	push_config_state();
	set_config_file("allegro.cfg");
	set_config_int("TUNA", "secret", unlockedSecret);
	pop_config_state();

	ExitMASkinG();
	return 0;
}
END_OF_MAIN();
